@@ -412,7 +412,7 @@ static void cmd_setpath(struct obex_session *os,
 		}
 
 		/* This is because OBEX_UnicodeToChar() accesses the string
-		   even if its size is zero */
+		 * even if its size is zero */
 		if (hlen == 0) {
 			os->name = g_strdup("");
 			break;
@@ -428,9 +428,9 @@ static void cmd_setpath(struct obex_session *os,
 	os->cmds->setpath(obex, obj);
 }
 
-gboolean os_prepare_get(struct obex_session *os, gchar *file, guint32 *size)
+int os_prepare_get(struct obex_session *os, gchar *file, guint32 *size)
 {
-	gint fd;
+	gint fd, err;
 	struct stat stats;
 
 	fd = open(file, O_RDONLY);
@@ -448,13 +448,14 @@ gboolean os_prepare_get(struct obex_session *os, gchar *file, guint32 *size)
 
 	*size = stats.st_size;
 
-	return TRUE;
+	return 0;
 
 fail:
+	err = -errno;
 	if (fd >= 0)
 		close(fd);
 
-	return FALSE;
+	return err;
 }
 
 static gint obex_write_stream(struct obex_session *os,
@@ -546,6 +547,8 @@ gint os_prepare_put(struct obex_session *os)
 		w = write(os->fd, os->buf + len, os->offset - len);
 		if (w < 0) {
 			gint err = errno;
+			error("write(%s): %s (%d)", path, strerror(errno),
+									errno);
 			if (err == EINTR)
 				continue;
 			else
@@ -925,17 +928,6 @@ static void obex_handle_destroy(gpointer user_data)
 	OBEX_Cleanup(obex);
 }
 
-static gboolean tty_reinit(gpointer data)
-{
-	struct server *server = data;
-
-	tty_init(server->services, server->folder, server->capability, server->devnode);
-
-	server_free(server);
-
-	return FALSE;
-}
-
 static gboolean obex_handle_input(GIOChannel *io,
 				GIOCondition cond, gpointer user_data)
 {
@@ -951,7 +943,7 @@ static gboolean obex_handle_input(GIOChannel *io,
 
 		os = OBEX_GetUserData(obex);
 		if (os->server->devnode)
-			g_idle_add(tty_reinit, os->server);
+			tty_closed();
 
 		return FALSE;
 	}
